/*
 * Copyright 1992, John F. Haugh II
 * All rights reserved.
 *
 * Permission is granted to copy and create derivative works for any
 * non-commercial purpose, provided this copyright notice is preserved
 * in all copies of source code, or included in human readable form
 * and conspicuously displayed on all copies of object code or
 * distribution media.
 */

#include <sys/types.h>
#include <fcntl.h>
#include "is_types.h"

#ifndef lint
static	char	sccsid[] = "@(#)is_serial.c	1.2	11:49:18	1/29/92";
#endif

extern	int	db_errno;

/*
 * isam_serial - create new serial number
 *
 * isam_serial() returns the next valid serial number for the current
 * ISAM file.
 */

unsigned long
isam_serial (file)
file_t	*file;
{
	unsigned long	serial;
	struct	flock	flock;
	off_t	offset = sizeof (long) + sizeof (off_t);

#ifndef	NO_LOCKING

	/*
	 * Lock the serial number location so concurrent updates work.
	 * This is a short-time lock, so block waiting on it.
	 */

	flock.l_type = F_WRLCK;
	flock.l_whence = 0;
	flock.l_start = sizeof (off_t);
	flock.l_len = sizeof (long);

	if (fcntl (file->f_key_fd, F_SETLKW, &flock))
		return (db_errno = DB_CANT_LOCK), -1;
#endif

	/*
	 * Seek to the location of the serial number in the ISAM file
	 * header.  It is after the ticket number.  Read in the serial
	 * number that was last handed out.
	 */

	if (lseek (file->f_key_fd, offset, 0) != offset) {
		db_errno = DB_EIO;
		goto fail;
	}
	if (read (file->f_key_fd, &serial, sizeof serial) != sizeof serial) {
		db_errno = DB_EIO;
		goto fail;
	}

	/*
	 * Increment the serial number (so we don't have duplicates) and
	 * then write it out.  This keeps the serial number updated for
	 * the next person to come along.
	 */

	serial++;

	if (lseek (file->f_key_fd, offset, 0) != offset) {
		db_errno = DB_EIO;
		goto fail;
	}
	if (write (file->f_key_fd, &serial, sizeof serial) != sizeof serial) {
		db_errno = DB_EIO;
		goto fail;
	}

#ifndef	NO_LOCKING

	/*
	 * Now unlock the file (possibly allowing the next waiter to get
	 * to the file), and give the user their unique serial number.
	 */

	flock.l_type = F_UNLCK;
	(void) fcntl (file->f_key_fd, F_SETLK, &flock);
#endif
	return serial;

fail:

	/* 
	 * Something went wrong after the file was locked.  Unlock the file
	 * and return an error indication.
	 */

#ifndef	NO_LOCKING
	flock.l_type = F_UNLCK;
	(void) fcntl (file->f_key_fd, F_SETLK, &flock);
#endif
	return -1;
}
